﻿//v 1.1

using UnityEngine;

namespace SmallHitbox
{
    public class SmallHitbox : ETGModule
    {
        GameObject hitboxDisplay = null;
        const float pixelsPerUnit = 16;

        public override void Init() {
        }

        public override void Start() {
            ETGModConsole.Commands.AddGroup("hitboxsize");
            ETGModConsole.Commands.GetGroup("hitboxsize").AddUnit("half", (_) => ChangeHitboxDimensions(Option.Half));
            ETGModConsole.Commands.GetGroup("hitboxsize").AddUnit("pixel", (_) => ChangeHitboxDimensions(Option.Pixel));
            ETGModConsole.Commands.GetGroup("hitboxsize").AddUnit("default", (_) => ChangeHitboxDimensions(Option.Default));
            ETGModConsole.Commands.GetGroup("hitboxsize").AddUnit("display", (_) => ChangeHitboxDimensions(Option.Display));
        }

        private void ChangeHitboxDimensions(Option option) {
            PlayerController player = GameManager.Instance.PrimaryPlayer;
            SpeculativeRigidbody speculativeRigidbody = player.specRigidbody;
            PixelCollider collider = speculativeRigidbody.HitboxPixelCollider;


            IntVector2 center = GetCenter(player);
            switch (option) {
                case Option.Half:
                    ChangeSizeAndOffset(collider, 4, 5, center);
                    break;
                case Option.Pixel:
                    ChangeSizeAndOffset(collider, 1, 1, center);
                    break;
                case Option.Default:
                    IntVector2 defaultSize = GetDefaultDimensions(player);
                    ChangeSizeAndOffset(collider, defaultSize.x, defaultSize.y, center);
                    break;
                case Option.Display:
                    DisplayHitbox(player);
                    break;
            }


            LogHitboxInfo(collider);
        }

        private void DisplayHitbox(PlayerController player) {
            PixelCollider collider = GetHitbox(player);
            string displayerName = "HitboxDisplay";

            if (hitboxDisplay == null) {
                hitboxDisplay = GameObject.CreatePrimitive(PrimitiveType.Cube);
                hitboxDisplay.name = displayerName;
                hitboxDisplay.transform.SetParent(player.transform);
            } else {
                hitboxDisplay.SetActive(!hitboxDisplay.activeSelf);
            }

            if (player.transform.Find(displayerName) == null) {
                hitboxDisplay.transform.SetParent(player.transform);
            }

            UpdateDisplay(collider);
        }

        private void UpdateDisplay(PixelCollider collider) {
            hitboxDisplay.transform.localScale = new Vector3(
                collider.Dimensions.x / pixelsPerUnit,
                collider.Dimensions.y / pixelsPerUnit,
                1f);
            Vector3 offset = new Vector3(
                collider.Offset.x + collider.Dimensions.x * 0.5f,
                collider.Offset.y + collider.Dimensions.y * 0.5f,
                -16f)
                / pixelsPerUnit;
            hitboxDisplay.transform.localPosition = offset;
        }

        private static PixelCollider GetHitbox(PlayerController player) {
            return player.specRigidbody.HitboxPixelCollider;
        }

        private void ChangeSizeAndOffset(PixelCollider collider, int newWidth, int newHeight, IntVector2 center) {
            collider.Dimensions = new IntVector2(newWidth, newHeight);
            collider.m_offset = new IntVector2(center.x - newWidth / 2, center.y - newHeight / 2);
            if (hitboxDisplay != null) {
                UpdateDisplay(collider);
            }
            ETGModConsole.Log("Changed hitbox size");
        }

        private static void LogHitboxInfo(PixelCollider collider) {
            ETGModConsole.Log("Hitbox is: " + collider.Dimensions.x + "x" + collider.Dimensions.y);
            //ETGModConsole.Log("Offset is: " + collider.Offset);
        }

        private static IntVector2 GetCenter(PlayerController player) {
            IntVector2 dimensions = GetDefaultDimensions(player);
            IntVector2 offset = GetDefaultOffset(player);
            return new IntVector2(offset.x + dimensions.x / 2, offset.y + dimensions.y / 2);
        }

        private static IntVector2 GetDefaultDimensions(PlayerController player) {
            //ETGModConsole.Log(player.name);
            switch (player.name) {
                case "PlayerCosmonaut(Clone)":
                    return new IntVector2(8, 16);
                default:
                    return new IntVector2(8, 10);
            }
        }

        private static IntVector2 GetDefaultOffset(PlayerController player) {
            //ETGModConsole.Log(player.name);
            switch (player.name) {
                case "PlayerGuide(Clone)":
                    return new IntVector2(9, 5);
                case "PlayerRobot(Clone)":
                    return new IntVector2(6, 5);
                case "PlayerConvict(Clone)":
                    return new IntVector2(7, 5);
                case "PlayerRogue(Clone)":
                    return new IntVector2(8, 5);
                case "PlayerBullet(Clone)":
                    return new IntVector2(10, 5);
                case "PlayerGunslinger(Clone)":
                    return new IntVector2(11, 5);
                case "PlayerMarine(Clone)":
                    return new IntVector2(8, 6);
                case "PlayerEevee(Clone)":
                    return new IntVector2(9, 5);
                case "PlayerLamey(Clone)":
                    return new IntVector2(8, 5);
                case "PlayerNinja(Clone)":
                    return new IntVector2(6, 5);
                case "PlayerCosmonaut(Clone)":
                    return new IntVector2(7, 3);
                default:
                    return new IntVector2(8, 5);
            }
        }

        public override void Exit() {
        }

        private enum Option
        {
            Half, Pixel, Default, Display
        }
    }
}
